// SNESpad class
// Version 1.0
// Kerry High

#include <iostream.h>
#include <conio.h>
#include "snespad.h"
#define SNES_PWR (128+64+32+16+8)
#define SNES_CLK 1  // base+0
#define SNES_LAT 2  // base+0
#define SIN (negate?((inp(base+1)&snes_din)?1:0):((inp(base+1)&snes_din)?0:1))

// New snespad, base=LPT1
SNESpad::SNESpad()
{
   setlpt(1);
   return;
}

// New snespad, set base by lpt#
SNESpad::SNESpad(int lpt_number)
{
   setlpt(lpt_number);
   return;
}

// Set base by lpt#: 1=0x378 2=0x278 3=0x3BC
void SNESpad::setlpt(int lpt_number)
{
   switch(lpt_number)
   {
   case 1: setbase(0x378); break;
   case 2: setbase(0x278); break;
   case 3: setbase(0x3bc); break;
   default: 
      cerr << "SNESpad: LPT" << lpt_number << " invalid!" << endl;
      cerr << "Defaulting to LPT1" << endl;
      setbase(0x378);
   }
   return;
}

// Set to any base
void SNESpad::setbase(int port_number)
{
   base=port_number;
   return;
}

// Read pad number pad_number
// Pads are numbered from 0 to 4
// Original C code by Earle F. Philhower, III.
void SNESpad::readpad(int pad_number)
{
   const int DIN[5] = {64,32,16,8,128};
   const int NEGATE[5]={0, 0, 0,0,  1};

   int snes_din=DIN[pad_number];
   int negate=NEGATE[pad_number];

   outp(base, SNES_PWR+SNES_CLK); // Power up!
   outp(base, SNES_PWR+SNES_LAT+SNES_CLK); // Latch it!

   outp(base, SNES_PWR+SNES_CLK);
   snes.b = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.y = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.select = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.start = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.up = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.down = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.left = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.right = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.a = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.x = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.l = SIN;
   outp(base, SNES_PWR);
   outp(base, SNES_PWR+SNES_CLK);
   snes.r = SIN;
   outp(base, 0); // Power it down
}

// Return 1 if button pressed at last read, 0 otherwise
int SNESpad::up()
{
   return snes.up;
}

int SNESpad::down()
{
   return snes.down;
}

int SNESpad::right()
{
   return snes.right;
}

int SNESpad::left()
{
   return snes.left;
}

int SNESpad::select()
{
   return snes.select;
}

int SNESpad::start()
{
   return snes.start;
}

int SNESpad::a()
{
   return snes.a;
}

int SNESpad::b()
{
   return snes.b;
}

int SNESpad::x()
{
   return snes.x;
}

int SNESpad::y()
{
   return snes.y;
}

int SNESpad::l()
{
   return snes.l;
}

int SNESpad::r()
{
   return snes.r;
}
